/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "project.h"
#include "cmsis_os2.h"
#include "kdrv_cmsis_core.h"

#include "kdrv_ddr.h"
#include "kdrv_system.h"
#include "kdrv_pinmux.h"

#include "kmdw_memory.h"
#include "kmdw_console.h"
#include "kmdw_display.h"


#define CAMERA_RGB_IDX              (0)
#define CAMERA_NIR_IDX              (1)

//project.h -> board.h
//#define LCDC_WIDTH                  (640)
//#define LCDC_HEIGHT                 (480)

static uint32_t pinmux_array[PIN_NUM] = PINMUX_ARRAY;

void example_dp_open(uint32_t width, uint32_t height, uint32_t pixelformat)
{
    struct video_input_params params;
  
    params.input_fmt = pixelformat;
    params.input_xres = width;
    params.input_yres = height;
  
    kmdw_video_renderer_open(&params);
    kmdw_video_renderer_set_camera(CAMERA_RGB_IDX);
    kmdw_display_set_pen_rgb565(BLACK, 4);
    kmdw_video_renderer_buffer_initialize(&params);
    kmdw_video_renderer_start();
}

/**
 * @brief main, main dispatch function
 */
int main(void)
{
    SystemCoreClockUpdate(); // System Initialization
    osKernelInitialize();    // Initialize CMSIS-RTOS

    /* below is some primiary system init settings */
    kdrv_system_init();                 // primary system init
    kdrv_system_init_ncpu();            // this is a must for ddr
    kdrv_ddr_system_init(DDR_INIT_ALL); // enable ddr

    /* below is some middleware init settings */
    kmdw_ddr_init(DDR_HEAP_BEGIN, DDR_HEAP_END);    // for memory alloc from ddr
    kdrv_uart_initialize(); // for log printing
    kdrv_uart_console_init(COMM_PORT_ID_0, COMM_UART_BAUDRATE_115200, NULL);  // enable console mdw for printing

    kdrv_pinmux_initialize(PIN_NUM, pinmux_array);
    
    /* init the display */
    kmdw_display_test_pattern_gen(TRUE);
    kmdw_display_initialize();

    /* init the application */
    example_dp_open(LCDC_WIDTH, LCDC_HEIGHT, V2K_PIX_FMT_RGB565);

    if (osKernelGetState() == osKernelReady)
    {
        osKernelStart();
    }

    while (1)
    {
    }
}
